
/**
 * 给予安全范围内的数据
 * @param {number} num 需要判断的数字
 * @param {number} max 最大值
 * @param {number} min 最小值
 * @returns {number} 校验后的数字
 */
 function checkNumber(num, max, min) {
    return Math.min(Math.max(num, min), max)
}

/**
 * 将表单格式的字符串反序列化为对象
 * @param {string} query 表单格式的数据字符串
 * @returns {{[key:string]:any}}
 */
function parseQueryString(query) {
    var parts = query.split('&');
    var params = {};
    for (var i = 0, ii = parts.length; i < ii; ++i) {
        var param = parts[i].split('=');
        var key = param[0].toLowerCase();
        var value = param.length > 1 ? param[1] : null;
        params[decodeURIComponent(key)] = decodeURIComponent(value);
    }
    return params;
}

/**
 * 获取query中的参数
 * @param {string} name 参数名称
 * @returns {string} 相应参数的值
 */
function getURLParameter(name) {
    return decodeURIComponent((new RegExp('[?|&]' + name + '=' + '([^&;]+?)(&|#|;|$)').exec(location.search) || [, ""])[1].replace(/\+/g, '%20')) || null;
}

/**
 * 获取印章默认值
 * @param {string} type 类型编号
 * @returns {number} 相应类型的默认值
 */
function getSealDefaultSize(type) {
    var DefaultSize = {
        "2": frzSealDefaultSize,
        "6": agentSealDefaultSize,
        "7": sxqmSealDefaultSize,
        "3": contractzSealDefaultSize
    }

    return Number(DefaultSize[type] || sealDefaultSize) 
}

var pageNum = 1,
    certSerialNum = null,
    //页面展示大小和pdf真实页面大小之比
    scale,
    //选中的印章图片base64
    selectedSealImgBase64 = null,
    /** @type {string} 选中的印章图片id */
    selectedSealId = null,
    /** @type {string} 选中的印章类型 */
    selectedSealType = null,
    //应用系统
    appid = "TP",
    //服务器地址
    serverurl = "http://ztbsign.icinfo.cn",
    //密钥
    secret = "633c0cb6f45b11e7971e00188b839ae8",
    //签名值
    signdata,
    //二维码base64
    picdata,
    //轮询时的uuid
    uuid,
    //二维码失效时间
    timeout,
    //layer序号
    index,
    //改变前scale
    preScale = 1,
    //改变后scale
    aftScale = 1,
    //签章类型
    signType,
    /** 页面是否正在缩放 */
    isZoom,
    /** 页面缩放时不画签章，但是把要画的页码放进数组中，缩放完成后立即画 */
    zoomPageArr = [],
    /** 
     * @typedef Seal 印章信息
     * @property {string} picdata 印章的图片的base64
     * @property {string} sealid 印章的id
     * @property {string} sealname 印章的名称
     * @property {string} sealtype 印章的类型编码 0：公章 2：未知 3：未知 6：未知 7：手绘印章
     */
    /** @type {Seal[]}  印章数组 */
    sealInfoArray,
    //弹出层返回结果
    result,
    startPage,
    endPage,
    pageArray,
    /** @type {number} 文档总页数 */
    totalPages,
    /** 公章默认大小 */
    sealDefaultSize = 120,
    /** 法人章默认大小 */
    frzSealDefaultSize = 120,
    /** 合同章默认大小 */
    contractzSealDefaultSize = 120,
    /** 手写签名默认大小 */
    sxqmSealDefaultSize = 120,
    /** 经办人章默认大小 */
    agentSealDefaultSize = 120,
    //关键字坐标数组
    keyWordPosition,
    //骑缝章的位置信息
    QFDataList = new Array(),
    /**
     * @typedef RealPDF 后端传入的pdf真实数据详情
     * @property {number} iPageNo 页码
     * @property {number} iPageSize 页码真实宽度
     * @property {number} iPageHeight 页码真实高度
     */
    /** @type {RealPDF[]} pdf真实页面宽度数组 */
    realWidthArray,
    /**
     * @typedef RealPageInfo pdf 真实页面宽高对象
     * @property {RealPDF['iPageSize']} pageWidth 页码真实宽度
     * @property {RealPDF['iPageHeight']} pageHeight 页码真实高度
     */
    /**
     * @type {{[key:number]:RealPageInfo}} pdf 真实页面宽高对象
     */
    realPageInfo = {},
    //多页签类型 1连续；2指定
    multiPageType,
    sealSignEnd,
    frSignEnd,
    agentSignEnd,
    sxqmSignEnd,
    contractSignEnd,
    /**
    * @type {number} 页面dom宽度相对于后端传入真实宽度的比值
    */
    pageScale,
    /** 
     * @typedef SignatureInfo 需要画的签章信息
     * @property {number} left x坐标
     * @property {number} top 处理前的y坐标
     * @property {number} page 页码
     * @property {number} realLeft 相对于后端真实页面宽度的真实x坐标
     * @property {number} realTop 相对于后端真实页面宽度的真实y坐标
     * @property {string} sealImgBase64 图片的base64
     * @property {string} sealId 签章id,
     * @property {number} timeStamp 签章时间戳标识,
     * @property {string} sealType 签章类型 0：公章 2：未知 3：未知 6：未知 7：手绘印章
     */
    /** @type {SignatureInfo[]} 需要画的签章信息数组 */
    signatureInfo = [], //需要画的签章信息数组
    /** @type {number} 当前进入可视窗口的页码 */
    scrollCurrentPage, // 当前进入可视窗口的页码
    metadat = [],
    /** 
     * @typedef SignInfo 签章详情
     * @property {number} llx x坐标(相对于pageWidth的)
     * @property {number} lly 处理前的y坐标(相对于pageWidth的)
     * @property {number} page 页码
     * @property {number} pageWidth 页面宽度（后端传入的真实宽度）
     * @property {number} pageHeight 页面高度（后端传入的真实高度）
     * @property {number} sealWidth 签章宽度（相对于pageWidth的宽度）
     * @property {number} height 签章高度（相对于pageHeight的高度）
     * @property {string} sealId 签章id,
     * @property {string} timeStamp 签章时间戳标识,
     * @property {string} sealType 签章类型 0：公章 2：未知 3：未知 6：未知 7：未知
     */
    /**
     * @type {SignInfo[]} 签章位置,id等信息;点击保存签章时,处理后传给后端
     */
    saveList = []; // 签章位置,id等信息;点击保存签章时,处理后传给后端
$(function () {
    var interval = setInterval(function () {
        if (PDFViewerApplication.initialized) {
            //添加页面缩放事件
            PDFViewerApplication.eventBus.on('scalechange', function (e) {
                isZoom = true
                debounceScaleSeal(e.scale)
            });
            PDFViewerApplication.eventBus.dispatch('scalechanged', { source: PDFViewerApplication, value: '1' })
            clearInterval(interval);
        }
    }, 1000);


    //自由签章模式下签章按钮
    $("#localSign").click(function () {
        sign();
    });

    //当前页章
    $("#singlePageSign").click(function () {
        signType = "singlePage";
        clientIsExist();
    });

    //多页章
    $("#multiPageSign").click(function () {
        signType = "multiPage";
        clientIsExist();
    });

    //骑缝章
    $("#QFSign").click(function () {
        signType = "QF";
        clientIsExist();
    });

    //关键字章
    $("#keyWordSign").click(function () {
        signType = "keyWord";
        clientIsExist();
    });

    //修改印章大小
    $("#changeSealSize").click(function () {
        doEnChangeSealSize();
    });

    //清空印章
    $("#removeSeal").click(function () {
        mConfirm("确认要刷新页面吗", function () {
            var sealImg = $("div[name='freeSignSeal'],div[name='QFSeal']");
            if (sealImg != undefined) {
                sealImg.remove();
            }
            saveList = []
            signatureInfo = []
            QFDataList = []
            mTips("页面已刷新");
        })
    });
});
/**
 * 删除操作时去 saveList 删除对应的签章信息
 * @param {any} row 单条需要删除的数据
 */
function deleteSign(row) {
    var index = saveList.findIndex(function (item) {
        return item.page == row.page && item.sealId == row.sealId  &&
        item.timeStamp == row.timeStamp
    })

    if (index > -1) {
        saveList.splice(index, 1)
    }
}

function drow (page) {
    // 遍历需要画的签章信息数组
    signatureInfo.filter(function (item) {
        return item.page == page
    }).forEach(function (item) {
        var options = {
            sealId: item.sealId,
            sealImgBase64: item.sealImgBase64,
            pageNum: page,
            top: item.top,
            left: item.left,
            timeStamp: item.timeStamp,
            realSealLeft: item.realLeft,
            realSealTop: item.realTop,
            type: item.type,
            selectedSealType: item.selectedSealType,
            continuity: true
        }
        // 缩放引起的页面滚动，可能会引起画签章，缩放时有300延时处理缩放比例，所以此处延时需要大于300（缩放延时）
        // setTimeout(function () {
            doCoverMultiPageSealImg(options)
        // }, 301)
    })

    // 从signatureInfo数组遍历删除已画在页面上的签章信息
    signatureInfo = signatureInfo.filter(function (item) {
        return item.page != page
    })
}
document.getElementById('viewerContainer').addEventListener('scroll', function () {

    
    /** @type {number} 当前进入可视窗口的页码 */
    var currentPage = window.PDFViewerApplication.page
    // 如果当前页码不是第一次进入,则退出
    if (scrollCurrentPage == currentPage) {
        return
    }
    if(isZoom) {
        zoomPageArr.push(currentPage)
        return
    }
    // 第一次进入,记录,防止在同一页码多次触发下面的循环(每次进入改页面都会触发一次下面的循环)
    scrollCurrentPage = currentPage
    var len = signatureInfo.length
    // 判断需要画的签章信息数组是否有内容，有内容则需要画签章
    if (len) {
        // 如果需要画签章
        drow(currentPage)
    }
})

/**
 * 提示框
 * @param {string} msg 提示信息
 * @param {()=>void} func 回调函数
 */
function mTips(msg, func) {
    layer.msg(msg, {
        //icon: 0, //0:! 1:对号 2：×
        time: 3500 //2秒关闭（如果不配置，默认是3秒）
    }, function () {
        if (func && typeof (func) == "function") {
            (func)();
        }
    });
}

/**
 * 弹窗
 * @param {string} msg 提示信息
 * @param {()=>void} func 回调函数
 */
function mAlert(msg, func) {
    layer.alert(msg, {
        icon: typeof (func) === 'number' ? func : 0,
        title: '提示'
    }, function (index) {
        if (func && typeof (func) == "function") {
            (func)();
        }
        layer.close(index);
    });
}

/**
 * 确认提示框
 * @param {string} msg 提示信息
 * @param {()=>void} success 回调函数
 */
function mConfirm(msg, success) {
    layer.confirm(msg,
        { icon: 3, title: '提示' },
        function (index) {
            if (success && typeof (success) == "function") {
                (success)(index);
            }
            layer.close(index);
        });
}


/** 
 * 载入中动画
 * @param {string=} msg 提示文案
 */
function showloading(msg) {
    var myMsg = msg || "正在处理，请稍等...";

    return layer.msg(myMsg, {
        icon: 16,
        shade: [0.5, '#f5f5f5'],
        scrollbar: false,
        time: 100000
    });
}
/**
 * @description: url追加参数
 * @param {*} uri url
 * @param {*} key 参数名
 * @param {*} value 参数值
 * @return {*} void
 */
 function updateQueryStringParameter(uri, key, value) {
    if (!value) {
        return uri;
    }
    var re = new RegExp("([?&])" + key + "=.*?(&|$)", "i");
    var separator = uri.indexOf("?") !== -1 ? "&" : "?";
    if (uri.match(re)) {
        return uri.replace(re, "$1" + key + "=" + value + "$2");
    } else {
        return uri + separator + key + "=" + value;
    }
}
/** 关闭动画 */
function hiddenLoading(index) {
    layer.close(index);
}

/** 判断是否有单页签章、多页签章 */
function isExistSeal() {
    var sealImg = $("div[name='freeSignSeal']");
    return !!(saveList.length || sealImg.length);
}

/** 判断是否有骑缝章 */
function isExistQFSeal() {
    var sealImg = $("div[name='QFSeal']");
    return !!sealImg.length;
}

/**
 * 判断客户端是否存在
 */
function clientIsExist() {
    var t2 = window.setTimeout(function () {
        mAlert("招投标控件初始化失败，请重新下载安装最新联连客户端！", function () {
            window.open("http://www.linksgood.com/page/client/topic/llianClient3");
        });
    }, 2000);
    //检测JSONP程序是否正常启动
    $.ajax({
        type: "get",
        async: false,
        url: "https://localhost:9819/lps/lps?method=1",
        dataType: "jsonp",
        jsonp: "callback",//传递给请求处理程序或页面的，用以获得jsonp回调函数名的参数名(一般默认为:callback)
        jsonpCallback: "myjsonpcall",//自定义的jsonp回调函数名称，默认为jQuery自动生成的随机函数名，也可以写"?"，jQuery会自动为你处理数据
        success: function (json) {
            //正常启动
            window.clearTimeout(t2);//去掉定时器
            //初始化realWidthArray
            if (realWidthArray === undefined) {
                totalPages = PDFViewerApplication.pagesCount;
                getRealPageWidth(1, totalPages);
            } else {
                //获取证书绑定的印章
                getSealList();
            }
        }
    });
}
function debounce(func, delay) {
    var timer;
    return function () {
        var context = this;// 注意 this 指向
        var args = arguments;// arguments中存着e
        if (timer) clearTimeout(timer);

        timer = setTimeout(function () {
            return func.apply(context, args)
        }, delay)
    }
}
var debounceScaleSeal = debounce(scaleSeal, 300)



//页面缩放时调整印章大小
function scaleSeal(scale) {
    // 获取到缩放比例
    aftScale = scale;
    $("div[name='freeSignSeal'],div[name='QFSeal']").each(function () {
        var $dom = $(this);
        var info = $dom.data('position');
        var width = info.realSealWidth * pageScale * scale;
        var height = info.realSealHeight * pageScale * scale;
        var top = info.realSealTop * scale * pageScale;
        var left = info.realSealLeft * scale * pageScale;
        var llx = left;
        var lly = top;
        $dom.css({ "width": width, "height": height, "top": top, "left": left });
        $dom.data('position', $.extend({}, info, {  llx: llx, lly: lly }));
    })
    signatureInfo.forEach(function (element) {
        element.top = element.realTop * scale * pageScale
        element.left = element.realLeft * scale * pageScale
    })
    // 当前页面相对于100%的缩放比例
    preScale = aftScale;
    // 缩放对应逻辑处理完成,画签章
    isZoom = false
    if(zoomPageArr.length) {
        for(var i = 0; i < zoomPageArr.length; i++ ) {
            var page = zoomPageArr.shift()
            drow(page)
        }
    }
}

//1获取印章列表
function getSealList() {
    $.ajax({
        type: "get",
        async: false,
        url: "https://localhost:9819/?method=60",
        data: {
            "serverurl": serverurl + "/api/eseal/getseallistbycert",
            "appid": appid,
            "secret": secret
        },
        dataType: "jsonp",
        jsonp: "callback",//传递给请求处理程序或页面的，用以获得jsonp回调函数名的参数名(一般默认为:callback)
        jsonpCallback: "myjsonpcall",//自定义的jsonp回调函数名称，默认为jQuery自动生成的随机函数名，也可以写"?"，jQuery会自动为你处理数据
        success: function (data) {
            if (data.retcode != 0) {
                mAlert(data.errinfo || "获取印章列表失败，请检测网络连接再重试。");
                return;
            }
            var mydata = data.retvalue;
            if (mydata.rslt == "fail") {
                mAlert(mydata.errormsg);
                return;
            }
            certSerialNum = mydata.certSn;
            sealInfoArray = mydata.data || [];
            if (signType == "singlePage") {
                doEnShowSealPage();
            } else if (signType == "QF") {
                // doEnSetPage();
                doGetQFSeal()
            } else if (signType == "multiPage") {
                doEnSetPage2();
            } else if (signType == "keyWord") {
                doEnSetKeyWord();
            }
        }
    });
}

//2开始签章
function sign() {
    if (!isExistSeal()) {
        if (isExistQFSeal()) {
            mAlert("请先点当前页签章后，再操作骑缝章。");
            return;
        } else {
            mAlert("您还没有签章，无需保存。");
            return;
        }
    }

    mConfirm("在您标记的印章位签章", function (layerIndex) {
        layer.close(layerIndex);

        /**
         * @typedef Pos 印章位置信息
         * @property {any} position 具体位置信息
         * @property {number} startPage 起始页
         * @property {number} endPage 结束页
         */
        /** 
         * 创建连续签章位置信息
         * @returns {Pos}
         */
        var creatPosition = function () {
            return {
                position: {},
                startPage: 0,
                endPage: 0
            }
        }

        /** 签章位置集合 */
        var positionList = [];
        /** 公章位置集合 */
        var sealPos = creatPosition();
        /** 法人章集合 */
        var frzPos = creatPosition();
        /** 合同章集合 */
        var contractPos = creatPosition();
        /** 手写签名集合 */
        var sxqmPos = creatPosition();
        /** 无名签章集合 */
        var agentPos = creatPosition();

        /**
         * 获取印象集合对象
         * @param {string} type 类型编号
         * @returns {Pos}
         */
        var getPos = function (type) {
            var Pos = {
                "2": frzPos,
                "6": agentPos,
                "7": sxqmPos,
                "3": contractPos
            }

            return Pos[type] || sealPos
        }
        /**
         * 获取类型结束页码
         * @param {string} type 类型编号
         * @returns {number} 页码
         */
        var getSignEnd = function (type) {
            var SignEnds = {
                "2": frSignEnd,
                "6": agentSignEnd,
                "7": sxqmSignEnd,
                "3": contractSignEnd
            }

            return SignEnds[type] || sealSignEnd
        }
        /**
         * @typedef ContinuousInfo 连续页详情
         * @property {number} startPage 起始页
         * @property {number} endPage 结束页
         * @property {number} x x轴距离
         * @property {number} y y轴距离
         * @property {string} sealid 印章id
         * @property {number} pageWidth 页面宽度
         * @property {number} sealWidth 印章宽度
         */
        /** @type {ContinuousInfo[]} */
        var continuousArray = [];
        /**
         * 创建连续页详情
         * @param {number} startPage 开始页
         * @param {number} endPage 结束页
         * @param {any} pos 印章position信息
         * @returns {ContinuousInfo} 连续页详情
         */
        var createContinuous = function (startPage, endPage, pos) {
            var continuous = new Object();
            continuous.startPage = startPage;
            continuous.endPage = endPage;
            continuous.x = Math.round(pos.x);
            continuous.y = Math.round(pos.y);
            continuous.sealid = pos.sealid;
            continuous.pageWidth = Math.round(pos.pageWidth);
            continuous.sealWidth = Math.round(pos.sealWidth);
            return continuous
        }
        /** 
         * 比较位置是否相同
         * @param {any} pos 位置对象
         * @param {any} target 需要比较的目标对象
         * @returns {boolean} 是否相等
         */
        var isEqualPosition = function (pos, target) {
            return pos.x == target.x && pos.y == target.y && pos.pageWidth == target.pageWidth && pos.sealWidth == target.sealWidth
        }
        var index;
        //获得印章的位置信息
        $("div[name='freeSignSeal']").each(function () {
            var selfPosition = $(this).data("position");
            if (selfPosition.signSelfType == 'multiple') {
                return
            }
            var position = new Object();
            position.x = Math.round(selfPosition.realSealLeft);
            position.y = realPageInfo[selfPosition.page].pageHeight - selfPosition.realSealTop - getSealDefaultSize(selfPosition.sealType);
            position.y = Math.floor(position.y);
            position.page = selfPosition.page;
            position.pageWidth = Math.round(realPageInfo[selfPosition.page].pageWidth)
            position.sealid = selfPosition.sealId;
            position.sealWidth = getSealDefaultSize(selfPosition.sealType);

            /** 相应类型的印章的对象 */
            var pos = getPos(selfPosition.sealType);
            var signEnd = getSignEnd(selfPosition.sealType);
            if (isEqualPosition(pos.position, position)) {
                //连续相同位置
                pos.endPage = position.page;
                //移除存在的元素
                index = positionList.indexOf(pos.position);
                if (index != -1) {
                    positionList.splice(index, 1);
                }
                if (position.page == signEnd) {
                    var continuous = createContinuous(pos.position, pos.endPage, position);
                    continuousArray.push(continuous);
                    pos.endPage = 0;
                }
            } else {
                if (pos.startPage != pos.endPage && pos.startPage != 0 && pos.endPage != 0) {
                    var continuous = createContinuous(pos.startPage, pos.startPage, pos.position);
                    continuousArray.push(continuous);
                    pos.endPage = 0;
                }
                positionList.push(position);
                pos.startPage = position.page;
            }
            pos.position = position;
        });
        saveList.forEach(function (item) {
            var position = {};
            position.x = Math.floor(item.llx);
            if (position.x < 0) {
                position.x = 0
            } else if (item.llx + Number(item.sealWidth) > item.pageWidth) {
                position.x = Math.floor(item.pageWidth - Number(item.sealWidth))
            }
            position.y = Math.floor(item.pageHeight - item.lly - Number(item.height))
            if (position.y < 0) {
                position.y = 0
            } else if (item.lly +  Number(item.height) > item.pageHeight) {
                position.y = Math.floor(item.pageHeight -  Number(item.height))
            }
            position.page = item.page;
            position.pageWidth = Math.floor(item.pageWidth);
            position.sealid = item.sealId;
            position.sealWidth = Math.floor(item.sealWidth);

            /** 相应类型的印章的对象 */
            var pos = getPos(item.sealType);
            var signEnd = getSignEnd(item.sealType);
            if (isEqualPosition(pos.position, position) && pos.position.page == position.page - 1) {
                //连续相同位置
                pos.endPage = position.page;
                //移除存在的元素
                for (var i = 0; i < positionList.length; i++) {
                    var item = positionList[i]
                    if (isEqualPosition(item, pos.position) && item.page == pos.position.page && item.sealid == pos.position.sealid) {
                        index = i
                        break
                    } else {
                        index = -1
                    }
                }
                if (index != -1) {
                    positionList.splice(index, 1);
                }
                if (position.page == signEnd) {
                    var continuous = createContinuous(pos.startPage, pos.endPage, position);
                    continuousArray.push(continuous);
                    pos.endPage = 0;
                }
            } else {
                if (pos.startPage != pos.endPage && pos.startPage != 0 && pos.endPage != 0) {
                    var continuous = createContinuous(pos.startPage, pos.endPage, pos.position);
                    continuousArray.push(continuous);
                    pos.endPage = 0;
                }
                positionList.push(position);
                pos.startPage = position.page;
            }
            pos.position = position;
        });
        var position = new Object();
        position.data = positionList;
        position.qfdata = QFDataList;
        position.continuousPage = continuousArray;
        var saveType = getURLParameter("saveType");
        var savePath = getURLParameter("savePath");
        var token = getURLParameter("token");
        var layerId = showloading("正在保存签章中，请勿做任何操作。");
        var params = parseQueryString(document.location.search.substring(1));
        var sourceFile = 'filepath' in params ? params.filepath : '';
        if (sourceFile.substr(0, 4) != 'http') {
            sourceFile = '';
        } else {
            var index = sourceFile.lastIndexOf("/");
            sourceFile = sourceFile.substring(index + 1, sourceFile.length);
            if (sourceFile.indexOf(".pdf") == -1) {
                sourceFile = sourceFile + ".pdf";
            }
        }
        $.ajax({
            type: "get",
            async: true,
            url: "http://localhost:9818/?method=61",
            data: {
                "serverurl": serverurl,
                "appid": appid,
                "secret": secret,
                "position": setParamBuffer(JSON.stringify(position), "position"),
                "sourceFile": sourceFile
            },
            dataType: "jsonp",
            jsonp: "callback",//传递给请求处理程序或页面的，用以获得jsonp回调函数名的参数名(一般默认为:callback)
            jsonpCallback: "myjsonpcall",//自定义的jsonp回调函数名称，默认为jQuery自动生成的随机函数名，也可以写"?"，jQuery会自动为你处理数据
            success: function (data) {
                hiddenLoading(layerId);
                if (data.retcode != 0) {
                    mAlert(data.errinfo);
                    return;
                } else {
                    if (saveType == null || saveType == '' || savePath == null || savePath == '' || token == null || token == '') {
                        mTips("签章保存成功，请务必检查文档中印章是否正确", function () {
                            var newurl = updateQueryStringParameter(
                                window.location.href,
                                "#page",
                                window.PDFViewerApplication.page
                            );
                            //向当前url添加参数，没有历史记录
                            window.history.replaceState(
                                {
                                    path: newurl,
                                },
                                "",
                                newurl
                            );
                            location.reload();
                        });
                    } else {
                        var uploadLoading = showloading("正在上传签章文件中，请勿做任何操作。");
                        $.ajax({
                            type: "get",
                            async: true,
                            url: "https://localhost:9819/?method=81",
                            data: {
                                "type": saveType,
                                "filepath": savePath,
                                "token": token,
                                "sourceFile": sourceFile
                            },
                            dataType: "jsonp",
                            jsonp: "callback",//传递给请求处理程序或页面的，用以获得jsonp回调函数名的参数名(一般默认为:callback)
                            jsonpCallback: "myjsonpcall",//自定义的jsonp回调函数名称，默认为jQuery自动生成的随机函数名，也可以写"?"，jQuery会自动为你处理数据
                            success: function (data) {
                                hiddenLoading(uploadLoading);
                                if (data.retcode != 0) {
                                    mAlert(data.errinfo);
                                    return;
                                } else {
                                    mTips("签章保存成功，请务必检查文档中印章是否正确", function () {
                                        var newurl = updateQueryStringParameter(
                                            window.location.href,
                                            "#page",
                                            window.PDFViewerApplication.page
                                        );
                                        //向当前url添加参数，没有历史记录
                                        window.history.replaceState(
                                            {
                                                path: newurl,
                                            },
                                            "",
                                            newurl
                                        );
                                        location.reload();
                                    });
                                }
                            }
                        })
                    }
                }
            }
        });
    })
}
/**
 * @description: 上传签章文件
 * @return {*} void
 */
function uploadSign() {
    var saveType = getURLParameter("saveType");
    var savePath = getURLParameter("savePath");
    var token = getURLParameter("token");
    var params = parseQueryString(document.location.search.substring(1));
    var sourceFile = "filepath" in params ? params.filepath : "";
    if (sourceFile.substr(0, 4) != "http") {
        sourceFile = "";
    } else {
        var index = sourceFile.lastIndexOf("/");
        sourceFile = sourceFile.substring(index + 1, sourceFile.length);
        if (sourceFile.indexOf(".pdf") == -1) {
            sourceFile = sourceFile + ".pdf";
        }
    }
    var uploadLoading = showloading("正在上传签章文件中，请勿做任何操作。");
    $.ajax({
        type: "get",
        async: true,
        url: "https://localhost:9819/?method=81",
        data: {
            type: saveType,
            filepath: savePath,
            token: token,
            sourceFile: sourceFile,
        },
        timeout: 10000,
        dataType: "jsonp",
        jsonp: "callback", //传递给请求处理程序或页面的，用以获得jsonp回调函数名的参数名(一般默认为:callback)
        jsonpCallback: "myjsonpcall", //自定义的jsonp回调函数名称，默认为jQuery自动生成的随机函数名，也可以写"?"，jQuery会自动为你处理数据
        success: function (data) {
            hiddenLoading(uploadLoading);
            if (data.retcode != 0) {
                mAlert(data.errinfo);
                return;
            } else {
                if (data.retvalue.ErrCode == -1) {
                    return mAlert(data.retvalue.ErrValue || "调用控件异常");
                }
                // alertTips('签章成功，请务必检查文档中印章是否正确', 'success')
                mTips("签章保存成功，请务必检查文档中印章是否正确", function () {
                    location.reload();
                });
            }
        },
        error: function (err) {
            return mAlert("调用控件异常");
        },
    });
}



//进入选择印章页面
function doEnShowSealPage() {
    result = null;
    index = layer.open({
        skin: 'custom-layer',
        type: 2,
        title: '选择印章',
        maxmin: false,
        area: ['700px', '500px'],
        content: 'certSealListRlt.html',
        end: function () {
            if (result == null || result == "close") {
                signType = "singlePage";
                return false;
            }
            if (result == 'success') {
                if (signType == "singlePage") {
                    doCoverSealImg();
                } else if (signType == "multiPage") {
                    var pageNum = pageArray[0]
                    var pageObj = $("#pageContainer" + pageNum);
                    var pageWidth = pageObj.width();
                    var pageHeight = pageObj.height();
                    var pdfFileRealWidth = getSpecificPageWidth(pageNum);
                    var scale = pageWidth / pdfFileRealWidth;
                    /** 印章的默认大小值 */
                    var defaultSize = getSealDefaultSize(selectedSealType);
                    var timeStamp = new Date().getTime();
                    saveList.push({
                        llx: 0,
                        lly: 0,
                        page: pageNum,
                        pageWidth: pageWidth,
                        pageHeight: pageHeight,
                        sealWidth: defaultSize,
                        sealId: selectedSealId,
                        sealType: selectedSealType,
                        height: defaultSize,
                        timeStamp: timeStamp,
                    })
                    var options = {
                        sealId: selectedSealId,
                        sealImgBase64: selectedSealImgBase64,
                        pageNum: pageNum,
                        top: 0,
                        left: 0,
                        timeStamp: timeStamp,
                        realSealLeft: 0,
                        realSealTop: 0
                    }
                    doCoverMultiPageSealImg(options);
                } else if (signType == "keyWord") {
                    doCoverKeyWordSealImg();
                    mAlert("关键字签章位置标记成功，请检查。");
                }

            }
        }
    });
}

//进入输入页码页面
function doEnSetPage() {
    result = null;
    index = layer.open({
        skin: 'custom-layer',
        type: 2,
        title: '输入页码',
        maxmin: false,
        area: ['500px', '320px'],
        content: 'setPageNum.html',
        end: function () {
            if (result == null || result == "close") {
                return false;
            }
            if (result == 'success') {
                doGetQFSeal();
            }
        }
    });
}

//进入输入页码页面
function doEnSetPage2() {
    result = null;
    index = layer.open({
        skin: 'custom-layer',
        type: 2,
        title: '输入页码',
        maxmin: false,
        area: ['500px', '320px'],
        content: 'setPageNum2.html',
        end: function () {
            if (result == null || result == "close") {
                signType = "singlePage";
                return false;
            }
            if (result == 'success') {
                doEnShowSealPage();
            }
        }
    });
}

//进入输入关键字页面
function doEnSetKeyWord() {
    result = null;
    index = layer.open({
        skin: 'custom-layer',
        type: 2,
        title: '输入关键字',
        maxmin: false,
        area: ['500px', '320px'],
        content: 'setKeyWord.html',
        end: function () {
            if (result == null || result == "close") {
                return false;
            }
            if (result == 'success') {
                doEnShowSealPage();
            }
        }
    });
}



//进入修改印章大小页面
function doEnChangeSealSize() {
    result = null;
    index = layer.open({
        skin: 'custom-layer',
        type: 2,
        title: '修改印章大小',
        maxmin: false,
        area: ['500px', '400px'],
        content: 'changeSealSize.html',
        end: function () {
            if (result == null || result == "close") {
                return false;
            }
            if (result == 'success') {
                var pageWidth = $("#pageContainer1").width()
                var pdfFileRealWidth = getSpecificPageWidth(1);
                var scale = pageWidth / pdfFileRealWidth;
                $("div[name='freeSignSeal']").each(function () {
                    var currentData = $(this).data('position')
                    var sealType = currentData.sealType;
                    /** 印章的默认大小值 */
                    var defaultSize = getSealDefaultSize(sealType);
                    /** 当前签章所在页面的宽度 */ 
                    var currentPageWidth = $(this).parent().width()
                    /** 当前签章所在页面的高度 */
                    var currentPageHeight = $(this).parent().height()
                    /** 当前页面所允许的最大left */
                    var maxLeft = currentPageWidth - defaultSize * pageScale * preScale
                    /** 当前页面所允许的最大Top */
                    var maxTop = currentPageHeight - defaultSize * pageScale * preScale
                    var sealWidth = scale * defaultSize;
                    var sealHeight = scale * defaultSize;
                    
                    var left = checkNumber(currentData.llx, maxLeft, 0)
                    var top =  checkNumber(currentData.lly, maxTop, 0)
                    /** 重新设置css样式 */
                    $(this).css({ width: sealWidth, height: sealHeight, left: left,top: top});
                    if(left != currentData.llx || top != currentData.lly) {
                        var resetPosition = {
                            realSealLeft: left / pageScale / preScale,
                            realSealTop: top / pageScale / preScale
                        }
                        /** 重置初始left，top */
                        $(this).data('position',$.extend({}, currentData, resetPosition))
                    }
                    $(this).data('position',$.extend({}, $(this).data('position'), {realSealWidth: defaultSize,realSealHeight: defaultSize}))
                });
                signatureInfo.forEach(function(element){
                   /** 签章大小 */ 
                    var defaultSize = getSealDefaultSize(element.sealType);
                    console.log(defaultSize)
                    /** 页面宽度 */
                    var pageWidth = realPageInfo[element.page].pageWidth * pageScale
                    /** 页面高度 */
                    var pageHeight = realPageInfo[element.page].pageHeight * pageScale
                    /** 最大left */
                    var maxLeft = pageWidth - defaultSize * pageScale * preScale
                    /** 最大top */
                    var maxTop = pageHeight - defaultSize * pageScale * preScale
                    /** 重新设置left top */
                    element.left = checkNumber(element.left, maxLeft, 0)
                    element.top = checkNumber(element.top, maxTop, 0)
                    /** 重新设置realLeft 和 realTop*/
                    element.realLeft = checkNumber(element.realLeft, maxLeft, 0)
                    element.realTop = checkNumber(element.realTop, maxTop, 0)
                })

                saveList.forEach(function (item) {
                    /** 印章的默认大小值 */
                    var defaultSize = getSealDefaultSize(item.sealType);
                    item.sealWidth = defaultSize
                    item.height = defaultSize
                    /** 后端传入的页面真实宽度 */
                    var pageWidth = item.pageWidth
                    /** 后端传入的页面真实高度 */
                    var pageHeight = item.pageHeight
                    /** 当前页面所允许的最大left */
                    var maxLeft = pageWidth - defaultSize
                    /** 当前页面所允许的最大Top */
                    var maxTop = pageHeight - defaultSize
                    /** 重新设置存储数据的left，top */
                    item.llx = checkNumber(item.llx, maxLeft, 0)
                    item.lly = checkNumber(item.lly, maxTop, 0)
                })
            }
        }
    });
}

//获取骑缝章图片
function doGetQFSeal() {
    var GZSealId;
    for (var i in sealInfoArray) {
        var sealInfo = sealInfoArray[i];
        if (sealInfo.sealtype == "0") {
            GZSealId = sealInfo.sealid;
        }
    }
    var param = {
        "appid": appid,
        "time": new Date().Format("yyyyMMddHHmmss"),
        "sealId": GZSealId,
        "startPage": '1',
        "endPage": PDFViewerApplication.pagesCount.toString()
    }
    $.ajax({
        type: "get",
        async: false,
        url: "https://localhost:9819/?method=66",
        data: {
            "serverurl": serverurl + "/api/eseal/doGetQFSeal",
            "secret": secret,
            "param": JSON.stringify(param)
        },
        dataType: "jsonp",
        jsonp: "callback",//传递给请求处理程序或页面的，用以获得jsonp回调函数名的参数名(一般默认为:callback)
        jsonpCallback: "myjsonpcall",//自定义的jsonp回调函数名称，默认为jQuery自动生成的随机函数名，也可以写"?"，jQuery会自动为你处理数据
        success: function (data) {
            if (data.retcode == "0") {
                var retvalue = data.retvalue;
                doCoverQFSealImg(retvalue);
                //保存骑缝章位置信息用于签章
                var qfDataObj = new Object();
                qfDataObj.blocksize = retvalue.blocksize;
                qfDataObj.lastsize = retvalue.lastsize;
                qfDataObj.sealid = GZSealId;
                qfDataObj.pagebegin = 1;
                qfDataObj.pageend = PDFViewerApplication.pagesCount;
                qfDataObj.sealWidth = sealDefaultSize;
                QFDataList.push(qfDataObj);
                mAlert("骑缝章签章位置标记成功，请检查。");
            } else {
                mAlert("骑缝章签署失败，请重试。");
            }
        }
    });
}

//添加印章印记
function doCoverSealImg() {
    if (selectedSealImgBase64 == '' || selectedSealImgBase64 == null) {
        mAlert("请先选择印章");
        return;
    }
    pageNum = PDFViewerApplication.page;
    var pageObj = $("#pageContainer" + pageNum);
    var pageWidth = pageObj.width();
    var pdfFileRealWidth = getSpecificPageWidth(pageNum);
    var scale = pageWidth / pdfFileRealWidth;

    /** 印章的默认大小值 */
    var defaultSize = getSealDefaultSize(selectedSealType);

    var sealWidth = scale * defaultSize;
    var sealHeight = scale * defaultSize;

    pageObj.createSign({
        cssClass: 'sealClass',
        sealId: selectedSealId,
        sealImgBase64: selectedSealImgBase64,
        page: pageNum,
        width: sealWidth,
        height: sealHeight,
        type: signType,
        sealType: selectedSealType,
        realSealLeft: 0,
        realSealTop: 0,
        realSealWidth: defaultSize,
        realSealHeight: defaultSize,
        signSelfType: 'single',
        endMove: function (data) {
            var movEendData = data.signObj.data('position')
            var realSealLeft = movEendData.llx / preScale / pageScale
            var realSealTop =  movEendData.lly / preScale / pageScale
            data.signObj.data('position', $.extend({}, movEendData, {  realSealTop: realSealTop, realSealLeft: realSealLeft }))
        }
    }, function (llx, lly) {
    });
    //跳转到页首
    PDFViewerApplication.page = pageNum;
    mAlert("请拖动印章至盖章处");
}

//添加多页印章印记
function doCoverMultiPageSealImg(options) {
    var sealId =  options.sealId
    var sealImgBase64 = options.sealImgBase64
    var pageNum = options.pageNum
    var top = options.top
    var left = options.left
    var timeStamp = options.timeStamp
    var mySignType =  'multiPage'
    var realSealLeft = options.realSealLeft || 0
    var realSealTop = options.realSealTop || 0
    if (sealImgBase64 == '' || sealImgBase64 == null) {
        mAlert("请先选择印章");
        return;
    }
    var pageObj = $("#pageContainer" + pageNum);
    /** 印章的默认大小值 */
    var defaultSize = getSealDefaultSize(options.selectedSealType || selectedSealType);
    var pageHeight = realPageInfo[options.pageNum].pageHeight * preScale * pageScale;
    var pageWidth = realPageInfo[options.pageNum].pageWidth * preScale * pageScale;
    var scaleWidth = defaultSize * pageScale * preScale

    if (top < 0) {
        top = 0;
    } else if (top >= pageHeight - scaleWidth) {
        top = pageHeight - scaleWidth;
    }
    if (left < 0) {
        left = 0;
    } else if (left >= pageWidth - scaleWidth) {
        left = pageWidth - scaleWidth;
    }
    /** 签章确认时的提示页码文案 */
    
    var tipPages;
    if (pageArray.length == 1) {
        tipPages = pageArray[0];
    } else {
        if (multiPageType == '2') {
            var pages = "";
            for (var i = 0; i < pageArray.length; i++) {
                if (pages == "") {
                    pages = pageArray[i];
                } else {
                    pages += ',' + pageArray[i];
                }
            }
            tipPages = pages;
        } else {
            tipPages = pageArray[0] + "-" + pageArray[pageArray.length - 1] ;
        }
    }
    pageObj.createSign({
        cssClass: 'sealClass',
        sealImgBase64: sealImgBase64,
        width: scaleWidth,
        height: scaleWidth,
        page: pageNum,
        sealId: sealId,
        type: options.type || signType,
        top: top,
        left: left,
        pageArray: pageArray,
        sealType: options.selectedSealType || selectedSealType,
        signSelfType: 'multiple',
        timeStamp: timeStamp,
        /** 签章真实宽高 */
        realSealWidth: defaultSize,
        realSealHeight: defaultSize,
        /** 签章真实left,Top */
        realSealLeft: realSealLeft,
        realSealTop: realSealTop,
        tipPages: tipPages,
        endMove: function (data) {
            var signObj = data.signObj;
            var movEendData = signObj.data('position')
            saveList.forEach(function (item) {
                if (item.sealId == movEendData.sealId && item.page == movEendData.page && item.timeStamp == movEendData.timeStamp) {
                    item.llx = movEendData.llx / pageScale / preScale
                    item.lly = movEendData.lly / pageScale / preScale
                    item.pageWidth = realPageInfo[movEendData.page].pageWidth
                    item.pageHeight = realPageInfo[movEendData.page].pageHeight
                }
            })
            signObj.data('position',$.extend({},movEendData,{
                realSealLeft: movEendData.llx / preScale / pageScale,
                realSealTop: movEendData.lly / preScale / pageScale
            }))
            if (signObj.attr("data-type") == "multiPage") {
                var content = "您确定在" + movEendData.tipPages + "页中放置该印章吗？";
                mConfirm(content, function () {
                    signType = "singlePage";
                    var multiPageInfo = signObj.data("multiPageInfo");
                    var multiPageArray = multiPageInfo.pageArray;
                    var sealId = multiPageInfo.sealId;
                    for (var i = 1; i < multiPageArray.length; i++) {
                        var page = multiPageArray[i]
                        signatureInfo.push($.extend({}, multiPageInfo, {
                            left: movEendData.llx,
                            top: movEendData.lly,
                            realLeft: movEendData.llx / preScale / pageScale,
                            realTop: movEendData.lly / preScale  / pageScale,
                            page: multiPageArray[i],
                            timeStamp: timeStamp,
                            type: 'singlePage',
                            selectedSealType: movEendData.sealType,
                        }))
                        
                        saveList.push({
                            llx: movEendData.llx / pageScale / preScale,
                            lly: movEendData.lly / pageScale / preScale,
                            page: page,
                            pageWidth: realPageInfo[page].pageWidth,
                            pageHeight: realPageInfo[page].pageHeight,
                            sealWidth: defaultSize,
                            height: defaultSize,
                            sealId: sealId,
                            sealType: movEendData.sealType,
                            timeStamp: timeStamp,
                        })
                    }
                    signObj.attr("data-type", "singlePage");
                })

            }
        }
    }, function (llx, lly) {
    });
    //跳转到页首
    if (signType == "multiPage" && !options.continuity) {
        PDFViewerApplication.page = Number(pageNum);
        mAlert("请拖动印章至盖章处");
    }
}

//批量添加骑缝章
function doCoverQFSealImg(retvalue) {
    var sealInfoArray = retvalue.blockimginfo;
    doCoverQFBlock(sealInfoArray);
    sealInfoArray = retvalue.lastimginfo;
    doCoverQFBlock(sealInfoArray);
}

//添加骑缝章
function doCoverQFBlock(sealInfoArray) {
    for (var i = 0; i < sealInfoArray.length; i++) {
        var sealInfo = sealInfoArray[i];
        var pageArray = sealInfo.page;
        for (var j = 0; j < pageArray.length; j++) {
            var page = pageArray[j];
            var pageObj = $("#pageContainer" + page);
            //页面的真实大小
            var pageWidth = realPageInfo[page].pageWidth;
            var pageHeight = realPageInfo[page].pageHeight;
            //计算真实pdf中的大小
            // var scale = pageWidth / getSpecificPageWidth(page);

            /** 印章的默认大小值 */
            // var defaultSize = getSealDefaultSize(selectedSealType);
            var defaultSize = sealDefaultSize;

            /** 每份的真实宽度 */
            var sealWidth = defaultSize / sealInfo.count;
            var sealHeight = defaultSize;
            
            pageObj.createSign({
                cssClass: 'sealClass',
                sealImgBase64: sealInfo.sealImgBase64,
                width: sealWidth * preScale * pageScale,
                height: sealHeight * preScale * pageScale,
                page: page,
                top: (pageHeight - sealHeight) / 2 * preScale * pageScale,//靠右居中显示
                left: (pageWidth - sealWidth) * preScale * pageScale,
                type: 'QF',
                sealType: selectedSealType,
                signSelfType: 'qf',
                realSealWidth: sealWidth,
                realSealHeight: defaultSize,
                realSealLeft: pageWidth - sealWidth,
                realSealTop: (pageHeight - sealHeight) / 2,
            }, function (llx, lly) {
            });
        }
    }
}

//添加关键字印章印记
function doCoverKeyWordSealImg() {
    if (selectedSealImgBase64 == '' || selectedSealImgBase64 == null) {
        mAlert("请先选择印章");
        return;
    }
    for (var i = 0; i < keyWordPosition.length; i++) {
        var sealInfo = keyWordPosition[i];
        pageNum = sealInfo.page;
        var pageObj = $("#pageContainer" + pageNum);
        var pageWidth = pageObj.width();
        var scale = pageWidth / sealInfo.pageWidth;

        /** 印章的默认大小值 */
        var defaultSize = getSealDefaultSize(selectedSealType);

        var sealWidth = scale * defaultSize;
        var sealHeight = scale * defaultSize;

        var top = sealInfo.y * scale;
        top = pageObj.height() - top - sealHeight / 2;
        var left = sealInfo.x * scale - sealWidth / 2;
        pageObj.createSign({
            cssClass: 'sealClass',
            sealImgBase64: selectedSealImgBase64,
            width: sealWidth,
            height: sealHeight,
            page: pageNum,
            sealId: selectedSealId,
            type: signType,
            top: top,
            left: left,
            sealType: selectedSealType,
            signSelfType: 'key',
            endMove: function () {
            }
        }, function (llx, lly) {
        });
    }
}

//获取特定页面范围的真实页面大小
function getRealPageWidth(startPage, endPage) {
    var params = parseQueryString(document.location.search.substring(1));
    var sourceFile = 'filepath' in params ? params.filepath : '';
    if (sourceFile.substr(0, 4) != 'http') {
        sourceFile = '';
    } else {
        var index = sourceFile.lastIndexOf("/");
        sourceFile = sourceFile.substring(index + 1, sourceFile.length);
        if (sourceFile.indexOf(".pdf") == -1) {
            sourceFile = sourceFile + ".pdf";
        }
    }
    $.ajax({
        type: "get",
        async: false,
        url: "https://localhost:9819/?method=64",
        data: {
            "startPage": startPage,
            "endPage": endPage,
            "sourceFile": sourceFile
        },
        dataType: "jsonp",
        jsonp: "callback",//传递给请求处理程序或页面的，用以获得jsonp回调函数名的参数名(一般默认为:callback)
        jsonpCallback: "myjsonpcall",//自定义的jsonp回调函数名称，默认为jQuery自动生成的随机函数名，也可以写"?"，jQuery会自动为你处理数据
        success: function (data) {
            if (data.retcode != 0) {
                mAlert(data.errinfo, function () {
                    if (data.retcode == -5) {
                        window.open("http://www.linksgood.com/page/client/topic/llianClient3");
                    }
                });
                return;
            } else {
                realWidthArray = data.retvalue.pageWidth || [];
                realWidthArray.forEach(function (item) {
                    realPageInfo[item.iPageNo] = {
                        pageHeight: Math.round(item.iPageHeight),
                        pageWidth: Math.round(item.iPageSize)
                    }
                })
                // PDFViewerApplication.pdfViewer._currentScale 该值为进入页面时页面默认的缩放比例
                pageScale = $("#pageContainer1").width() / realPageInfo[1].pageWidth / PDFViewerApplication.pdfViewer._currentScale;
                //获取证书绑定的印章
                getSealList();
            }
        }
    });
}

/**
 * 获取指定页码的页面真实宽度
 * @param {number} pageNum 页码
 * @returns {number} 页面真实宽度
 */
function getSpecificPageWidth(pageNum) {
    var pageInfo = realWidthArray.find(function (item) { return item.iPageNo === pageNum }) || { iPageSize: 0 }
    return pageInfo.iPageSize
}

//当调用jsonp参数过长时采用设置缓存的方式
function setParamBuffer(paramValue, paramName) {
    // return paramValue;
    //设置每次缓存的字符长度
    var bufferLength = 2000;
    if (paramValue.length <= bufferLength) {
        return paramValue;
    } else {
        var index = 0;
        var totalCallCount = Math.ceil(paramValue.length / bufferLength);
        for (var callCount = 0; callCount < totalCallCount; callCount++) {
            index = bufferLength * callCount;
            var buffer = paramValue.substr(index, bufferLength);
            $.ajax({
                type: "get",
                async: false,
                url: "https://localhost:9819/?method=21",
                data: {
                    "callcount": callCount,
                    "paramname": paramName,
                    "paramvalue": buffer
                },
                dataType: "jsonp",
                jsonp: "callback",//传递给请求处理程序或页面的，用以获得jsonp回调函数名的参数名(一般默认为:callback)
                jsonpCallback: "myjsonpcall",//自定义的jsonp回调函数名称，默认为jQuery自动生成的随机函数名，也可以写"?"，jQuery会自动为你处理数据
                success: function (data) {
                }
            });
        }
        return "buffer";
    }
}